% Computing the whitening -logF transformation from the natural images

clear; 

% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scale = .7;  % scale of Gabors, length = 17 pixels
norient = 15;  % number of orientations
T = 0.;        % threshold for responses
Upperbound = 300.; % the range we compute -logF
binsize = .2;  % binsize for computing F

% TRAINING EXAMPLES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Iname = dir('rand85_127\*.jpg');  % folder that contains natural images as q(I)
nimage = size(Iname, 1);
Isize = zeros(nimage, 2); 
for i = 1 : nimage
    tmpIm = imread(['rand85_127' '\' Iname(i).name]);
    if size(tmpIm,3) == 3
        tmpIm = rgb2gray(tmpIm);
    end
    I{i} = imresize(double(tmpIm), 1); 
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for i = 1 : nimage
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the training images to be the same

% FILTERING IMAGES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
allfiltered = applyfilter(I, allfilter, T);  % filter training images


binnum = floor(Upperbound/binsize)+1;  % binnumbers
histog = zeros(binnum, 1);  % store F  



% MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mex Chist.c;   % compile C code
'histograming'
Chist(nimage, norient, allfiltered, h, sx, sy, binsize, binnum, histog);


% RESULTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure; 
r = (0:(binnum-1))*binsize;
nlf = -log(histog); 
plot(r, nlf); % plot coding gain curve
bnf = binnum; 
bsf = binsize; 
save 'nlf' nlf bnf bsf; 



